/* machine.h
 *
 * Copyright (C) 2017 Timothy Richard Musson
 *
 * Email: Tim Musson <trmusson@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 */


#ifndef HAVE_MACHINE_H
#define HAVE_MACHINE_H

#include "keyboard.h"
#include "video.h"


typedef struct _Machine Machine;


Machine*  machine_new (const char *progname,
                       const char *fname_profile,
                       int ram_size,
                       const char *fname_load,
                       const char *fname_tape,
                       xbool_t quiet,
                       xbool_t fullscreen);

void      machine_free (Machine *m);
xbool_t    machine_start (Machine *m, int entry_point);

void      machine_reset (Machine *m);
void      machine_toggle_stepping (Machine *m);
void      machine_step_now (Machine *m);

Profile*  machine_get_profile (Machine *m);
Keyboard* machine_get_keyboard (Machine *m);
Video*    machine_get_video (Machine *m);


#endif

