/* opcodes.h
 *
 * Copyright (C) 2017, 2020 Timothy Richard Musson
 *
 * Copying and distribution of this file, with or without modification,
 * are permitted provided the copyright notice and this notice are
 * preserved.
 *
 * 6502 microprocessor opcodes
 *
 */

#ifndef _HAVE_OPCODES_H
#define _HAVE_OPCODES_H


#define OP_ADC      0x69
#define OP_ADC_A    0x6D
#define OP_ADC_AX   0x7D
#define OP_ADC_AY   0x79
#define OP_ADC_Z    0x65
#define OP_ADC_ZX   0x75
#define OP_ADC_ZY   -1
#define OP_ADC_I    -1
#define OP_ADC_IX   0x61
#define OP_ADC_IY   0x71
 
#define OP_AND      0x29
#define OP_AND_A    0x2D
#define OP_AND_AX   0x3D
#define OP_AND_AY   0x39
#define OP_AND_Z    0x25
#define OP_AND_ZX   0x35
#define OP_AND_ZY   -1
#define OP_AND_I    -1
#define OP_AND_IX   0x21
#define OP_AND_IY   0x31

#define OP_ASL_ACC  0x0A
#define OP_ASL_A    0x0E
#define OP_ASL_AX   0x1E
#define OP_ASL_Z    0x06
#define OP_ASL_ZX   0x16

#define OP_BIT_A    0x2C
#define OP_BIT_Z    0x24

#define OP_BEQ_R    0xF0
#define OP_BCC_R    0x90
#define OP_BCS_R    0xB0
#define OP_BMI_R    0x30
#define OP_BNE_R    0xD0
#define OP_BPL_R    0x10
#define OP_BVC_R    0x50
#define OP_BVS_R    0x70

#define OP_BRK_IMP  0x00

#define OP_CLC_IMP  0x18
#define OP_CLD_IMP  0xD8
#define OP_CLI_IMP  0x58
#define OP_CLV_IMP  0xB8

#define OP_CMP      0xC9
#define OP_CMP_A    0xCD
#define OP_CMP_AX   0xDD
#define OP_CMP_AY   0xD9
#define OP_CMP_Z    0xC5
#define OP_CMP_ZX   0xD5
#define OP_CMP_ZY   -1
#define OP_CMP_I    -1
#define OP_CMP_IX   0xC1
#define OP_CMP_IY   0xD1

#define OP_CPX      0xE0
#define OP_CPX_A    0xEC
#define OP_CPX_Z    0xE4

#define OP_CPY      0xC0
#define OP_CPY_A    0xCC
#define OP_CPY_Z    0xC4

#define OP_DEC      -1
#define OP_DEC_A    0xCE
#define OP_DEC_AX   0xDE
#define OP_DEC_AY   -1
#define OP_DEC_Z    0xC6
#define OP_DEC_ZX   0xD6
#define OP_DEC_ZY   -1
#define OP_DEC_I    -1
#define OP_DEC_IX   -1
#define OP_DEC_IY   -1

#define OP_DEX_IMP  0xCA

#define OP_DEY_IMP  0x88

#define OP_EOR      0x49
#define OP_EOR_A    0x4D
#define OP_EOR_AX   0x5D
#define OP_EOR_AY   0x59
#define OP_EOR_Z    0x45
#define OP_EOR_ZX   0x55
#define OP_EOR_ZY   -1
#define OP_EOR_I    -1
#define OP_EOR_IX   0x41
#define OP_EOR_IY   0x51

#define OP_INC      -1
#define OP_INC_A    0xEE
#define OP_INC_AX   0xFE
#define OP_INC_AY   -1
#define OP_INC_Z    0xE6
#define OP_INC_ZX   0xF6
#define OP_INC_ZY   -1
#define OP_INC_I    -1
#define OP_INC_IX   -1
#define OP_INC_IY   -1

#define OP_INX_IMP  0xE8

#define OP_INY_IMP  0xC8

#define OP_JMP_A    0x4C
#define OP_JMP_I    0x6C

#define OP_JSR_A    0x20

#define OP_LDA      0xA9
#define OP_LDA_A    0xAD
#define OP_LDA_AX   0xBD
#define OP_LDA_AY   0xB9
#define OP_LDA_Z    0xA5
#define OP_LDA_ZX   0xB5
#define OP_LDA_ZY   -1
#define OP_LDA_I    -1
#define OP_LDA_IX   0xA1
#define OP_LDA_IY   0xB1

#define OP_LDX      0xA2
#define OP_LDX_A    0xAE
#define OP_LDX_AX   -1
#define OP_LDX_AY   0xBE
#define OP_LDX_Z    0xA6
#define OP_LDX_ZX   -1
#define OP_LDX_ZY   0xB6
#define OP_LDX_I    -1
#define OP_LDX_IX   -1
#define OP_LDX_IY   -1

#define OP_LDY      0xA0
#define OP_LDY_A    0xAC
#define OP_LDY_AX   0xBC
#define OP_LDY_AY   -1
#define OP_LDY_Z    0xA4
#define OP_LDY_ZX   0xB4
#define OP_LDY_ZY   -1
#define OP_LDY_I    -1
#define OP_LDY_IX   -1
#define OP_LDY_IY   -1

#define OP_LSR_ACC  0x4A
#define OP_LSR      -1
#define OP_LSR_A    0x4E
#define OP_LSR_AX   0x5E
#define OP_LSR_AY   -1
#define OP_LSR_Z    0x46
#define OP_LSR_ZX   0x56
#define OP_LSR_ZY   -1
#define OP_LSR_I    -1
#define OP_LSR_IX   -1
#define OP_LSR_IY   -1

#define OP_NOP_IMP  0xEA
 
#define OP_ORA      0x09
#define OP_ORA_A    0x0D
#define OP_ORA_AX   0x1D
#define OP_ORA_AY   0x19
#define OP_ORA_Z    0x05
#define OP_ORA_ZX   0x15
#define OP_ORA_ZY   -1
#define OP_ORA_I    -1
#define OP_ORA_IX   0x01
#define OP_ORA_IY   0x11
 
#define OP_PHA_IMP  0x48
#define OP_PLA_IMP  0x68
#define OP_PHP_IMP  0x08
#define OP_PLP_IMP  0x28

#define OP_ROL_ACC  0x2A
#define OP_ROL_A    0x2E
#define OP_ROL_AX   0x3E
#define OP_ROL_Z    0x26
#define OP_ROL_ZX   0x36

#define OP_ROR_ACC  0x6A
#define OP_ROR_A    0x6E
#define OP_ROR_AX   0x7E
#define OP_ROR_Z    0x66
#define OP_ROR_ZX   0x76

#define OP_RTI_IMP  0x40

#define OP_RTS_IMP  0x60
 
#define OP_SBC      0xE9
#define OP_SBC_A    0xED
#define OP_SBC_AX   0xFD
#define OP_SBC_AY   0xF9
#define OP_SBC_Z    0xE5
#define OP_SBC_ZX   0xF5
#define OP_SBC_ZY   -1
#define OP_SBC_I    -1
#define OP_SBC_IX   0xE1
#define OP_SBC_IY   0xF1

#define OP_SEC_IMP  0x38
#define OP_SED_IMP  0xF8
#define OP_SEI_IMP  0x78

#define OP_STA      -1
#define OP_STA_A    0x8D
#define OP_STA_AX   0x9D
#define OP_STA_AY   0x99
#define OP_STA_Z    0x85
#define OP_STA_ZX   0x95
#define OP_STA_ZY   -1
#define OP_STA_I    -1
#define OP_STA_IX   0x81
#define OP_STA_IY   0x91
 
#define OP_STX      -1
#define OP_STX_A    0x8E
#define OP_STX_AX   -1
#define OP_STX_AY   -1
#define OP_STX_Z    0x86
#define OP_STX_ZX   -1
#define OP_STX_ZY   0x96
#define OP_STX_I    -1
#define OP_STX_IX   -1
#define OP_STX_IY   -1
 
#define OP_STY      -1
#define OP_STY_A    0x8C
#define OP_STY_AX   -1
#define OP_STY_AY   -1
#define OP_STY_Z    0x84
#define OP_STY_ZX   0x94
#define OP_STY_ZY   -1
#define OP_STY_I    -1
#define OP_STY_IX   -1
#define OP_STY_IY   -1

#define OP_TAX_IMP  0xAA
#define OP_TAY_IMP  0xA8
#define OP_TXA_IMP  0x8A
#define OP_TYA_IMP  0x98
#define OP_TSX_IMP  0xBA
#define OP_TXS_IMP  0x9A


#endif

